/*
 * Written by Richard Rogers <rprogers@uw.edu>
 *
 * Copyright (C) 2015 Jeff Bilmes
 * Licensed under the Open Software License version 3.0
 * See COPYING or http://opensource.org/licenses/OSL-3.0
 */
 
#include <iostream>

#include <stdlib.h>
#include <stdio.h>

#include "json/json.h"

#define GMTK_ASC_PROTOCOL_COOKIE    "GMTa\n"
#define GMTK_ASC_PROTOCOL_VERSION   "0000\n"

void countOpenParens(std::string s, int &open_parens) {
  for (char const *c = s.c_str(); *c; ++c) {
    switch (*c) {
    case '[': open_parens += 1; break;
    case ']': open_parens -= 1; break;
    default: break;
    }
  }
}

int main(int argc, char *argv[]) {
  if (argc != 3) {
    fprintf(stderr,"\n%s m n\n\n\tm is # of continuous elements\n"
                   "\tn is # of discrete elements\n", argv[0]);
    return 1;
  }
  int m = atoi(argv[1]), n = atoi(argv[2]);
  printf("%s%s%d %d\n", 
	 GMTK_ASC_PROTOCOL_COOKIE, GMTK_ASC_PROTOCOL_VERSION, m, n);

  Json::Reader reader;
  Json::Value root;
  std::string line;
  bool current_segment_empty = true;
  for ( ; !( std::cin.eof() || std::cin.fail() ) ; ) {
    std::string json_text;
    int open_parens = 0;
    do {
      std::getline(std::cin, line);
      json_text += line;
      countOpenParens(line, open_parens);
    } while (open_parens > 0);
    if (0 == json_text.length()) continue;
    reader.parse(json_text, root, false);
    if (root.size() > 0) {
      if (root[0].size() != m) {
	fprintf(stderr, "ERROR: expected %d floats but got %d\n", 
		m, root[0].size());
	return 2;
      }
      if (root[1].size() != n) {
	fprintf(stderr, "ERROR: expected %d ints but got %d\n", 
		n, root[1].size());
	return 2;
      }
      printf("F");
      for (int i=0; i < m; ++i) {
	printf(" %f", root[0][i].asFloat());
      }
      for (int i=0; i < n; ++i) {
	printf(" %u", root[1][i].asUInt());
      }
      printf("\n");
    } else {
      printf ("E\n");
    }
    fflush(stdout);
    if (root.size() > 0) {
      // current segment has at least 1 frame
      current_segment_empty = false;
    } else { 
      // end of current segment
      if (current_segment_empty) {
	// end of stream
	break;
      }
      // reset for next segment
      current_segment_empty = true;
    }
  }
  return 0;
}
