/*
 * Written by Richard Rogers <rprogers@uw.edu>
 *
 * Copyright (C) 2015 Jeff Bilmes
 * Licensed under the Open Software License version 3.0
 * See COPYING or http://opensource.org/licenses/OSL-3.0
 */

#include <iostream>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "json/json.h"

#define GMTK_ASC_COOKIE_LENGTH    6
#define GMTK_ASC_VERSION_LENGTH   6
#define GMTK_ASC_PROTOCOL_COOKIE  "GMTa\n"
#define GMTK_ASC_PROTOCOL_VERSION "0000\n"

int main(int argc, char *argv[]) {
  if (argc < 3 || argc > 4) {
    fprintf(stderr,"\n%s [-lf] m n\n\n\tm is # of continuous elements\n"
                   "\tn is # of discrete elements\n", argv[0]);
    return 1;
  }

  Json::Writer *writer = new Json::FastWriter();
  int arg_num = 1;
  if (strncmp(argv[arg_num], "-lf", 4) == 0) {
    writer = new Json::StyledWriter();
    arg_num += 1;
  }
  int m = atoi(argv[arg_num++]), n = atoi(argv[arg_num++]);
  double *continuous = new double[m];
  int    *discrete   = new int[n];

  char cookie[GMTK_ASC_COOKIE_LENGTH];
  char version[GMTK_ASC_VERSION_LENGTH]; // protocol version #

  if (fgets(cookie, GMTK_ASC_COOKIE_LENGTH, stdin) != cookie) {
    fprintf(stderr,"ERROR: ASCII stream did not begin with 'GMTa\\n'\n");
    return 5;
  }
  if (strcmp(cookie, GMTK_ASC_PROTOCOL_COOKIE)) {
    fprintf(stderr, "ERROR: ASCII stream did not begin with 'GMTa\\n'\n");
    return 5;
  }
  if (fgets(version, GMTK_ASC_VERSION_LENGTH, stdin) != version) {
    fprintf(stderr, "ERROR: ASCII stream couldn't read protocol version\n");
    return 5;
  }
  if (strcmp(version, GMTK_ASC_PROTOCOL_VERSION) > 0) {
    version[GMTK_ASC_VERSION_LENGTH-2] = 0;
    fprintf(stderr, "ERROR: input ASCII stream version %s is newer than this "
	    "implementation's version %s",
	    version, GMTK_ASC_PROTOCOL_VERSION);
    return 5;
  }
  unsigned nCont, nDisc;
  if (fscanf(stdin, "%u %u", &nCont, &nDisc) != 2) {
    fprintf(stderr, "ERROR: ASCII stream failed to read the number of "
	    "continuous and discreate features");
    return 5;
  }
  if (nCont != m) {
    fprintf(stderr, "ERROR: ASCII stream contains %u continuous features, "
	    "but expected %u", nCont, m);
    return 5;
  }
  if (nDisc != n) {
    fprintf(stderr, "ERROR: ASCII stream contains %u discrete features, "
	    "but expected %u", nDisc, n);
    return 5;
  }

  bool empty_segment = true;
  for (;;) {
    char tag;
    if (scanf(" %c ", &tag) != 1) {
      fprintf(stderr, "ERROR: unable to read frame tag\n");
      return 2;
    }
    switch (tag) {
    case 'F':
    case 'f':
      empty_segment = false;
      for (int i=0; i < m; ++i) {
	if (scanf(" %lf ", continuous+i) != 1) {
	  fprintf(stderr, "ERROR: unable to read continuous value\n");
	  return 3;
	}
      }
      for (int i=0; i < n; ++i) {
	if (scanf(" %d ", discrete+i) != 1) {
	  fprintf(stderr, "ERROR: unable to read discrete value\n");
	  return 4;
	}
      }
      {
	Json::Value root;
	root.resize(2);
	root[0] = Json::Value(Json::arrayValue);
	root[0].resize(m);
	for (int i=0; i < m; ++i) root[0][i] = continuous[i];
	root[1] = Json::Value(Json::arrayValue);
	for (int i=0; i < n; ++i) root[1][i] = discrete[i];
	std::string output = writer->write(root);
	std::cout << output;
      }
      break;
    case 'E':
    case 'e':
      {
	Json::Value root(Json::arrayValue);
	std::cout << root << '\n';
      }
      if (empty_segment) return 0;
      empty_segment = true;
      break;
    default:
      fprintf(stderr, "ERROR: got tag '%c' but expected 'F' or 'E'\n", tag);
      return 5;
    }
    std::cout.flush();
  }
}
