
%option nounput
%option noinput
%option noyywrap

%{

/*
 * Written by Richard Rogers <rprogers@uw.edu>
 *
 * Copyright (C) 2015 Jeff Bilmes
 * Licensed under the Open Software License version 3.0
 * See COPYING or http://opensource.org/licenses/OSL-3.0
 */

#include <stdlib.h>
#include <iostream>
using namespace std;

#include "json/json.h"

#define register 

#define YY_INPUT(buf,result,max_size) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		}

bool first_segment = true;
Json::Writer *writer;
Json::Value frame(Json::arrayValue);
int idx;

%}

%x VARNAME
%x FRAMENUM
%x VITVAL

%%

Segment" "[0-9]+.*	{ if (!first_segment) cout << writer->write(frame); cout.flush();
                          first_segment=false; }
Ptn-[0-9]+" "[PCE]'?:	idx = 0; BEGIN(VARNAME);
<VARNAME>" "
<VARNAME>[^ (]+	        frame[idx]["variableName"] = yytext;
<VARNAME>"("		BEGIN(FRAMENUM);

<FRAMENUM>[0-9]+	frame[idx]["frame"] = atoi(yytext);
<FRAMENUM>")="		BEGIN(VITVAL);

<VITVAL>[0-9]+		frame[idx++]["MPEvalue"] = atoi(yytext);
<VITVAL>[^0-9,\n][^,\n]*	frame[idx++]["MPEvalue"] = yytext;
<VITVAL>","		BEGIN(VARNAME);
<VITVAL>\n		cout << writer->write(frame); cout.flush(); frame.resize(0); BEGIN(INITIAL);

.
\n

%%

int main(int argc, char *argv[]) {
  writer = new Json::FastWriter();
  if (argc == 2 && strncmp(argv[1], "-lf", 4) == 0) {
    writer = new Json::StyledWriter();
  }
  yylex();
  frame.resize(0);
  cout << writer->write(frame) << writer->write(frame);
  return 0;
}
